﻿// Copyright © Microsoft Corporation. 
// This source is subject to the Microsoft Source License for Silverlight Controls (March 2008 Release).
// Please see http://go.microsoft.com/fwlink/?LinkID=111693 for details.
// All other rights reserved. 

using System;
using System.Collections.ObjectModel; 
using System.Windows; 
using System.Windows.Controls;
using Microsoft.VisualStudio.TestTools.UnitTesting; 

#if WPF
public class SilverlightTest { } 
#else
using Microsoft.Silverlight.Testing;
#endif 
 
#if WPF && !WPFIMPLEMENTATION
namespace WPF 
#else
namespace System.Windows.Controls.Test
#endif 
{
    /// <summary>
    /// Unit tests for System.Windows.Controls.ListBox 
    /// </summary> 
    [TestClass]
    public sealed partial class ListBoxTest : SilverlightControlTest 
    {
        private string[] testStrings = new string[] { "hello", "world" };
        private int[] testIntegers = new int[] { 1, 2, 3 }; 
        private static ListBoxItem[] GetTestListBoxItems()
        {
            // Always return a new set of items because the previous items may 
            // still be in the visual tree somewhere 
            return new ListBoxItem[] { new ListBoxItem { Content = "ListBoxItemA" }, new ListBoxItem { Content = "ListBoxItemB" } };
        } 

        [TestMethod]
        public void Constructor() 
        {
            ListBox listBox = new ListBox();
            Assert.IsNotNull(listBox); 
        } 

        [TestMethod] 
        public void InitialValues()
        {
            ListBox listBox = new ListBox(); 
            Assert.AreEqual(-1, listBox.SelectedIndex);
            Assert.IsNull(listBox.SelectedItem);
            Assert.IsNotNull(listBox.SelectedItems); 
            Assert.AreEqual(0, listBox.SelectedItems.Count); 
            Assert.IsFalse(listBox.SelectedItems.IsReadOnly);
            Assert.AreEqual(SelectionMode.Single, listBox.SelectionMode); 
            Assert.IsFalse(ListBox.GetIsSelectionActive(listBox));
        }
 
        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void GetIsSelectionActiveWithNull() 
        { 
            ListBox.GetIsSelectionActive(null);
            // Threw because argument was null 
        }

        [TestMethod] 
        [ExpectedException(typeof(InvalidOperationException))]
        public void ChangeIsSelectionActive()
        { 
            ListBox listBox = new ListBox(); 
            listBox.SetValue(ListBox.IsSelectionActiveProperty, true);
            // Threw because IsSelectionActive is read-only 
        }

        [TestMethod] 
        public void AddStrings()
        {
            ListBox listBox = new ListBox(); 
            foreach (string s in testStrings) 
            {
                listBox.Items.Add(s); 
            }
            Assert.AreEqual(testStrings.Length, listBox.Items.Count);
            for (int i = 0; i < testStrings.Length; i++) 
            {
                Assert.AreEqual(testStrings[i], listBox.Items[i]);
            } 
        } 

        [TestMethod] 
        public void AddIntegers()
        {
            ListBox listBox = new ListBox(); 
            foreach (int i in testIntegers)
            {
                listBox.Items.Add(i); 
            } 
            Assert.AreEqual(testIntegers.Length, listBox.Items.Count);
            for (int i = 0; i < testIntegers.Length; i++) 
            {
                Assert.AreEqual(testIntegers[i], listBox.Items[i]);
            } 
        }

        [TestMethod] 
        public void AddAndClearListBoxItems() 
        {
            ListBoxItem[] testListBoxItems = GetTestListBoxItems(); 
            ListBox listBox = new ListBox();
            foreach (ListBoxItem item in testListBoxItems)
            { 
                listBox.Items.Add(item);
            }
            Assert.AreEqual(testListBoxItems.Length, listBox.Items.Count); 
            for (int i = 0; i < testListBoxItems.Length; i++) 
            {
                Assert.AreEqual(testListBoxItems[i], listBox.Items[i]); 
            }
            listBox.Items.Clear();
            Assert.AreEqual(0, listBox.Items.Count); 
        }

        [TestMethod] 
        public void BasicSelectedIndex() 
        {
            ListBox listBox = new ListBox(); 
            listBox.Items.Add(CreateNewObject());
            Assert.AreEqual(-1, listBox.SelectedIndex);
            listBox.SelectedIndex = 0; 
            Assert.AreEqual(0, listBox.SelectedIndex);
        }
 
        [TestMethod] 
        public void BasicSelectedItem()
        { 
            ListBox listBox = new ListBox();
            object obj = CreateNewObject();
            listBox.Items.Add(obj); 
            Assert.IsNull(listBox.SelectedItem);
            listBox.SelectedItem = obj;
            Assert.AreEqual(obj, listBox.SelectedItem); 
        } 

        [TestMethod] 
        [ExpectedException(typeof(InvalidOperationException))]
        public void BasicSelectedItems()
        { 
            ListBox listBox = new ListBox();
            object obj = CreateNewObject();
            listBox.Items.Add(obj); 
            listBox.SelectedItems.Add(obj); 
            // Threw because SelectedItems is only for multiple-selection mode
        } 

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))] 
        public void SelectedItemsSetValue()
        {
            ListBox listBox = new ListBox(); 
            listBox.SetValue(ListBox.SelectedItemsProperty, new object[0]); 
            // Threw because SelectedItems is read-only
        } 

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))] 
        public void BasicSelectionMode()
        {
            ListBox listBox = new ListBox(); 
            // SelectionMode setter is private because SelectionMode.Single is the only valid value 
            listBox.SetValue(ListBox.SelectionModeProperty, SelectionMode.Single);
            Assert.AreEqual(SelectionMode.Single, listBox.SelectionMode); 
            listBox.SetValue(ListBox.SelectionModeProperty,
#if WPF
                (SelectionMode)9);  // Invalid 
#else
                (SelectionMode)1);  // SelectionMode.Multiple
#endif 
            // Threw because an invalid value was specified 
        }
 
        [TestMethod]
        public void SelectionConsistency()
        { 
            ListBox listBox = new ListBox();

            // Add objA and select with SelectedIndex 
            object objA = CreateNewObject(); 
            listBox.Items.Add(objA);
            listBox.SelectedIndex = 0; 
            AssertSelection(listBox, 0, objA);

            // Add objB; leave objA selected 
            object objB = CreateNewObject();
            listBox.Items.Add(objB);
            AssertSelection(listBox, 0, objA); 
 
            // Select objB with SelectedIndex
            listBox.SelectedIndex = 1; 
            AssertSelection(listBox, 1, objB);

            // Select objA with SelectedItem 
            listBox.SelectedItem = objA;
            AssertSelection(listBox, 0, objA);
 
            // Select objB with SelectedItem 
            listBox.SelectedItem = objB;
            AssertSelection(listBox, 1, objB); 
        }

        private static void AssertSelection(ListBox listBox, int index, object item) 
        {
            Assert.AreEqual(index, listBox.SelectedIndex);
            Assert.AreEqual(item, listBox.SelectedItem); 
            Assert.AreEqual(1, listBox.SelectedItems.Count); 
            Assert.AreEqual(item, listBox.SelectedItems[0]);
        } 

        [TestMethod]
        public void IsSelectedForListBoxItem() 
        {
            ListBox listBox = new ListBox();
            ListBoxItem itemA = new ListBoxItem(); 
            ListBoxItem itemB = new ListBoxItem(); 
            listBox.Items.Add(itemA);
            Assert.IsFalse(itemA.IsSelected); 
            listBox.SelectedItem = itemA;
            Assert.IsTrue(itemA.IsSelected);
            listBox.Items.Add(itemB); 
            Assert.IsTrue(itemA.IsSelected);
            Assert.IsFalse(itemB.IsSelected);
            listBox.SelectedItem = itemB; 
            Assert.IsFalse(itemA.IsSelected); 
            Assert.IsTrue(itemB.IsSelected);
            listBox.SelectedItem = null; 
            Assert.IsFalse(itemA.IsSelected);
            Assert.IsFalse(itemB.IsSelected);
        } 

        [TestMethod]
        public void SelectionChangedForSelection() 
        { 
            ListBox listBox = new ListBox();
            object obj = CreateNewObject(); 
            bool fired = false;
            listBox.SelectionChanged += delegate(object sender, SelectionChangedEventArgs e)
            { 
                fired = true;
                Assert.IsNotNull(e.RemovedItems);
                Assert.AreEqual(0, e.RemovedItems.Count); 
                Assert.IsNotNull(e.AddedItems); 
                Assert.AreEqual(1, e.AddedItems.Count);
                Assert.AreEqual(obj, e.AddedItems[0]); 
            };
            listBox.Items.Add(obj);
            Assert.IsFalse(fired); 
            listBox.SelectedIndex = 0;
            Assert.IsTrue(fired);
        } 
 
        [TestMethod]
        public void SelectionChangedForSelectionRemoval() 
        {
            ListBox listBox = new ListBox();
            object obj = CreateNewObject(); 
            listBox.Items.Add(obj);
            listBox.SelectedIndex = 0;
            bool fired = false; 
            listBox.SelectionChanged += delegate(object sender, SelectionChangedEventArgs e) 
            {
                fired = true; 
                Assert.IsNotNull(e.RemovedItems);
                Assert.AreEqual(1, e.RemovedItems.Count);
                Assert.AreEqual(obj, e.RemovedItems[0]); 
                Assert.IsNotNull(e.AddedItems);
                Assert.AreEqual(0, e.AddedItems.Count);
            }; 
            listBox.SelectedIndex = -1; 
            Assert.IsTrue(fired);
        } 

        [TestMethod]
        public void SelectionChangedForChange() 
        {
            ListBox listBox = new ListBox();
            object objA = CreateNewObject(); 
            object objB = CreateNewObject(); 
            listBox.Items.Add(objA);
            listBox.Items.Add(objB); 
            listBox.SelectedIndex = 0;
            bool fired = false;
            listBox.SelectionChanged += delegate(object sender, SelectionChangedEventArgs e) 
            {
                fired = true;
                Assert.IsNotNull(e.RemovedItems); 
                Assert.AreEqual(1, e.RemovedItems.Count); 
                Assert.AreEqual(objA, e.RemovedItems[0]);
                Assert.IsNotNull(e.AddedItems); 
                Assert.AreEqual(1, e.AddedItems.Count);
                Assert.AreEqual(objB, e.AddedItems[0]);
            }; 
            listBox.SelectedIndex = 1;
            Assert.IsTrue(fired);
        } 
 
        [TestMethod]
        public void SelectionChangedForNoChangeIndex() 
        {
            ListBox listBox = new ListBox();
            listBox.Items.Add(CreateNewObject()); 
            listBox.SelectedIndex = 0;
            listBox.SelectionChanged += delegate { Assert.Fail(); };
            listBox.SelectedIndex = 0; 
        } 

        [TestMethod] 
        public void SelectionChangedForNoChangeItem()
        {
            ListBox listBox = new ListBox(); 
            object obj = CreateNewObject();
            listBox.Items.Add(obj);
            listBox.SelectedItem = obj; 
            listBox.SelectionChanged += delegate { Assert.Fail(); }; 
            listBox.SelectedItem = obj;
        } 

        [TestMethod]
        public void OnlySelectedItemRemoved() 
        {
            ListBox listBox = new ListBox();
            ObservableCollection<object> items = new ObservableCollection<object>(); 
            listBox.ItemsSource = items; 
            AddToPanelToRunOverrideMethods(listBox);
            items.Add(CreateNewObject()); 
            listBox.SelectedIndex = 0;
            bool fired = false;
            listBox.SelectionChanged += delegate { fired = true; }; 
            items.RemoveAt(0);
            Assert.AreEqual(-1, listBox.SelectedIndex);
            Assert.IsTrue(fired); 
        } 

        [TestMethod] 
        public void OnlySelectedItemCleared()
        {
            ListBox listBox = new ListBox(); 
            ObservableCollection<object> items = new ObservableCollection<object>();
            listBox.ItemsSource = items;
            AddToPanelToRunOverrideMethods(listBox); 
            items.Add(CreateNewObject()); 
            listBox.SelectedIndex = 0;
            bool fired = false; 
            listBox.SelectionChanged += delegate { fired = true; };
            items.Clear();
            Assert.AreEqual(-1, listBox.SelectedIndex); 
            Assert.IsTrue(fired);
        }
 
        [TestMethod] 
        public void SelectedItemRemoved()
        { 
            ListBox listBox = new ListBox();
            ObservableCollection<object> items = new ObservableCollection<object>();
            listBox.ItemsSource = items; 
            AddToPanelToRunOverrideMethods(listBox);
            items.Add(CreateNewObject());
            items.Add(CreateNewObject()); 
            listBox.SelectedIndex = 0; 
            bool fired = false;
            listBox.SelectionChanged += delegate { fired = true; }; 
            items.RemoveAt(0);
            Assert.AreEqual(-1, listBox.SelectedIndex);
            Assert.IsTrue(fired); 
        }

        [TestMethod] 
        public void SelectedItemMoved() 
        {
            ListBox listBox = new ListBox(); 
            ObservableCollection<object> items = new ObservableCollection<object>();
            listBox.ItemsSource = items;
            AddToPanelToRunOverrideMethods(listBox); 
            items.Add(CreateNewObject());
            listBox.SelectedIndex = 0;
            listBox.SelectionChanged += delegate { Assert.Fail(); }; 
            items.Insert(0, CreateNewObject()); 
            Assert.AreEqual(1, listBox.SelectedIndex);
        } 

        [TestMethod]
        public void SelectViaListBoxItemDoesNothing() 
        {
            ListBoxItem[] testListBoxItems = GetTestListBoxItems();
            ListBox listBox = new ListBox(); 
            listBox.ItemsSource = testListBoxItems; 
            testListBoxItems[1].IsSelected = true;
            Assert.AreEqual(-1, listBox.SelectedIndex); 
            Assert.AreEqual(null, listBox.SelectedItem);
        }
 
        [TestMethod]
        public void UnselectViaListBoxItemDoesNothing()
        { 
            ListBoxItem[] testListBoxItems = GetTestListBoxItems(); 
            ListBox listBox = new ListBox();
            listBox.ItemsSource = testListBoxItems; 
            listBox.SelectedIndex = 1;
            Assert.IsTrue(testListBoxItems[1].IsSelected);
            testListBoxItems[1].IsSelected = false; 
            Assert.AreEqual(1, listBox.SelectedIndex);
            Assert.AreEqual(testListBoxItems[1], listBox.SelectedItem);
        } 
 
        [TestMethod]
        public void AddListBoxItemWithIsSelectedTrue() 
        {
            ListBoxItem[] testListBoxItems = GetTestListBoxItems();
            ListBox listBox = new ListBox(); 
            ObservableCollection<object> items = new ObservableCollection<object>();
            listBox.ItemsSource = items;
            AddToPanelToRunOverrideMethods(listBox); 
            testListBoxItems[0].IsSelected = true; 
            items.Add(testListBoxItems[0]);
            Assert.AreEqual(0, listBox.SelectedIndex); 
            testListBoxItems[1].IsSelected = true;
            items.Add(testListBoxItems[1]);
            Assert.AreEqual(1, listBox.SelectedIndex); 
        }

        [TestMethod] 
        public void ItemsViaItemsSource() 
        {
            ListBox listBox = new ListBox(); 
            listBox.ItemsSource = testStrings;
            Assert.AreEqual(testStrings.Length, listBox.Items.Count);
            for (int i = 0; i < testStrings.Length; i++) 
            {
                Assert.AreEqual(testStrings[i], listBox.Items[i]);
            } 
        } 

#if !WPFIMPLEMENTATION  // NotifyListItemClicked 
        [TestMethod]
        [Asynchronous]
        public void NotifyListItemClickedItem() 
        {
            ListBoxItem[] testListBoxItems = GetTestListBoxItems();
            ListBox listBox = new ListBox(); 
            listBox.Items.Add(testListBoxItems[0]); 
#if WPF
            AddToPanelToRunOverrideMethods(listBox); 
#endif
            CreateAsyncTest(listBox, delegate
            { 
                listBox.NotifyListItemClicked(testListBoxItems[0]);
                Assert.IsTrue(testListBoxItems[0].IsSelected);
            }); 
        } 
#endif
 
#if !WPFIMPLEMENTATION  // NotifyListItemGotFocus/NotifyListItemLostFocus
        [TestMethod]
        public void NotifyListItemGotLostFocus() 
        {
            ListBoxItem[] testListBoxItems = GetTestListBoxItems();
            ListBox listBox = new ListBox(); 
            listBox.Items.Add(testListBoxItems[0]); 
            listBox.Items.Add(testListBoxItems[1]);
            AddToPanelToRunOverrideMethods(listBox); 

            listBox.NotifyListItemGotFocus(testListBoxItems[0]);
            Assert.IsFalse(testListBoxItems[0].IsSelected); 
            listBox.NotifyListItemLostFocus(testListBoxItems[0]);
            Assert.IsFalse(testListBoxItems[0].IsSelected);
            listBox.NotifyListItemGotFocus(testListBoxItems[1]); 
            Assert.IsFalse(testListBoxItems[0].IsSelected); 
#if WPF
            Assert.IsTrue(testListBoxItems[1].IsSelected); 
#endif
        }
#endif 

        [TestMethod]
        public void CallMeasureToInvokeOverrideMethods() 
        { 
            ListBox listBox = new ListBox();
            ObservableCollection<object> items = new ObservableCollection<object>(); 
            listBox.ItemsSource = items;
            AddToPanelToRunOverrideMethods(listBox);
 
            // Add and select ListBoxitem
            ListBoxItem itemA = new ListBoxItem();
            items.Add(itemA); 
            listBox.SelectedIndex = 0; 
            Assert.IsTrue(itemA.IsSelected);
 
            // Add and select object
            object objA = CreateNewObject();
            items.Add(objA); 
            listBox.SelectedIndex = 1;
            Assert.AreEqual(objA, listBox.SelectedItem);
 
            // Add and select ListBoxitem 
            ListBoxItem itemB = new ListBoxItem();
            items.Add(itemB); 
            listBox.SelectedIndex = 2;
            Assert.IsTrue(itemB.IsSelected);
 
            // Clear and verify
            items.Clear();
            Assert.IsFalse(itemA.IsSelected); 
            Assert.IsFalse(itemB.IsSelected); 
        }
 
        [TestMethod]
        public void ApplyTemplate()
        { 
            ListBox listBox = new ListBox();
            ApplyTemplateAndAddToPanel(listBox);
        } 
 
        [TestMethod]
        public void ItemTemplate() 
        {
            ListBox listBox = new ListBox();
            listBox.ItemsSource = testIntegers; 
            listBox.ItemTemplate = GetItemTemplate();
            ApplyTemplateAndAddToPanel(listBox);
        } 
 
        [TestMethod]
        [Asynchronous] 
        public void ScrollIntoViewVertical()
        {
            ListBox listBox = new ListBox(); 
            ListBoxItem[] listBoxItems = GetTestListBoxItems();
            listBox.ItemsSource = listBoxItems;
            listBox.Height = 10; 
#if WPF 
            ApplyTemplateAndAddToPanel(listBox);
#endif 
            CreateAsyncTest(listBox, delegate
            {
                listBox.ScrollIntoView(listBoxItems[1]); 
            });
        }
 
        [TestMethod] 
        [Asynchronous]
        public void ScrollIntoViewHorizontal() 
        {
            ListBox listBox = new ListBox();
            listBox.ItemsPanel = GetHorizontalItemsPanelTemplate(); 
            ListBoxItem[] listBoxItems = GetTestListBoxItems();
            listBox.ItemsSource = listBoxItems;
            listBox.Width = 10; 
#if WPF 
            ApplyTemplateAndAddToPanel(listBox);
#endif 
            CreateAsyncTest(listBox, delegate
            {
                listBox.ScrollIntoView(listBoxItems[1]); 
            });
        }
 
#if !WPFIMPLEMENTATION  // HelperListBox 

        [TestMethod] 
        public void GotAndLostFocus()
        {
            HelperListBox helperListBox = new HelperListBox(); 
            helperListBox.ItemsSource = GetTestListBoxItems();
            helperListBox.SelectedIndex = 0;
            helperListBox.CallOnGotFocus(); 
            helperListBox.CallOnLostFocus(); 
        }
 
#endif

        [TestMethod] 
        public void ItemContainerStyleBeforeItems()
        {
            ListBox listBox = new ListBox(); 
            Style style = GetItemsContainerStyle(); 
            listBox.ItemContainerStyle = style;
            listBox.ItemsSource = GetTestListBoxItems(); 
            AddToPanelToRunOverrideMethods(listBox);
            foreach(ListBoxItem listBoxItem in listBox.Items)
            { 
                Assert.AreEqual(style, listBoxItem.Style);
            }
        } 
 
#if !WPFIMPLEMENTATION  // WPF ListBox Style application differences
 
        [TestMethod]
        public void ItemContainerStyleAfterItems()
        { 
            ListBox listBox = new ListBox();
            listBox.ItemsSource = GetTestListBoxItems();
            AddToPanelToRunOverrideMethods(listBox); 
            Style style = GetItemsContainerStyle(); 
            listBox.ItemContainerStyle = style;
            foreach (ListBoxItem listBoxItem in listBox.Items) 
            {
                Assert.AreEqual(style, listBoxItem.Style);
            } 
        }

        [TestMethod] 
#if !WPF 
        [ExpectedException(typeof(Exception))]
#endif 
        public void StyleCanBeSetOnlyOnce()
        {
            FrameworkElement frameworkElement = new Border();  // FrameworkElement is abstract 
            frameworkElement.Style = GetItemsContainerStyle();
            frameworkElement.Style = GetItemsContainerStyle();
        } 
 
        [TestMethod]
#if !WPF 
        [ExpectedException(typeof(NotSupportedException))]
#endif
        public void ItemContainerStyleChanges() 
        {
            ListBox listBox = new ListBox();
            listBox.ItemsSource = GetTestListBoxItems(); 
            AddToPanelToRunOverrideMethods(listBox); 
            Style style = GetItemsContainerStyle();
            listBox.ItemContainerStyle = style; 
            foreach (ListBoxItem listBoxItem in listBox.Items)
            {
                Assert.AreEqual(style, listBoxItem.Style); 
            }
            Style style2 = GetItemsContainerStyle();
            listBox.ItemContainerStyle = style2; 
            foreach (ListBoxItem listBoxItem in listBox.Items) 
            {
                Assert.AreEqual(style2, listBoxItem.Style); 
            }
            listBox.ItemContainerStyle = null;
            foreach (ListBoxItem listBoxItem in listBox.Items) 
            {
                Assert.AreEqual(null, listBoxItem.Style);
            } 
        } 

#endif 

        private static void ApplyTemplateAndAddToPanel(ListBox listBox)
        {
#if WPF  // LB-SV-WPF: Changed assembly name
            string ListBoxTemplate =
                "<ControlTemplate " + 
                    "xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"" + 
                    " xmlns:x=\"http://schemas.microsoft.com/winfx/2006/xaml\" " +
                    "xmlns:controls=\"clr-namespace:WPF;assembly=LB-SV-WPF\">" + 
                    "<controls:ScrollViewer x:Name=\"ScrollViewerElement\">" +
                        "<ItemsPresenter />" +
                    "</controls:ScrollViewer>" + 
                "</ControlTemplate>";
            ControlTemplate template = ListBoxItemTest.LoadXamlFromString<ControlTemplate>(ListBoxTemplate);
            Assert.IsNotNull(template); 
            listBox.Template = template; 
#endif
            AddToPanelToRunOverrideMethods(listBox); 
        }

        private static ItemsPanelTemplate GetHorizontalItemsPanelTemplate() 
        {
            return ListBoxItemTest.LoadXamlFromString<ItemsPanelTemplate>("<ItemsPanelTemplate xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"><StackPanel Orientation=\"Horizontal\"/></ItemsPanelTemplate>");
        } 
 
        private static DataTemplate GetItemTemplate()
        { 
            return ListBoxItemTest.LoadXamlFromString<DataTemplate>("<DataTemplate xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"/>");
        }
 
        private static Style GetItemsContainerStyle()
        {
            return ListBoxItemTest.LoadXamlFromString<Style>("<Style xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"/>"); 
        } 

        private static void AddToPanelToRunOverrideMethods(ListBox listBox) 
        {
            // This method gets the ItemsControl.*Override overrides called by putting the ListBox in a Panel
            StackPanel stackPanel = new StackPanel(); 
            stackPanel.Children.Add(listBox);
            stackPanel.Measure(new Size { Width = 100, Height = 100 });
            stackPanel.Arrange(new Rect(0, 0, 100, 100)); 
        } 

        private static object CreateNewObject() 
        {
            //
            return new Canvas(); 
        }
    }
} 
